@@ -134,30 +134,28 @@ public:
               : QString()));
 
         if (query.lastError().isValid()) {
-            qDebug() << "Error: " << query.lastError();
+            qWarning() << "[Error at ResultSetPrivate::initQuery]: " << query.lastError();
         }
-
-        Q_ASSERT_X(query.isActive(), "ResultSet initQuery", "Query is not valid");
     }
 
     QString agentClause(const QString &agent) const
     {
         if (agent == QLatin1String(":any")) return QStringLiteral("1");
 
-        return "agent = '" + (
+        return QStringLiteral("agent = '") + (
                 agent == QLatin1String(":current") ? QCoreApplication::instance()->applicationName() :
                                       agent
-            ) + "'";
+            ) + QStringLiteral("'");
     }
 
     QString activityClause(const QString &activity) const
     {
         if (activity == QLatin1String(":any")) return QStringLiteral("1");
 
-        return "activity = '" + (
+        return QStringLiteral("activity = '") + (
                 activity == QLatin1String(":current") ? ActivitiesSync::currentActivity(activities) :
                                          activity
-            ) + "'";
+            ) + QStringLiteral("'");
     }
 
     inline QString starPattern(const QString &pattern) const
@@ -171,14 +169,29 @@ public:
     {
         if (urlFilter == QLatin1String("*")) return QStringLiteral("1");
 
-        return "resource LIKE '" + Common::starPatternToLike(urlFilter) + "' ESCAPE '\\'";
+        return QStringLiteral("resource LIKE '") + Common::starPatternToLike(urlFilter) + QStringLiteral("' ESCAPE '\\'");
     }
 
     QString mimetypeClause(const QString &mimetype) const
     {
         if (mimetype == QLatin1String(":any") || mimetype == QLatin1String("*")) return QStringLiteral("1");
 
-        return "mimetype LIKE '" + Common::starPatternToLike(mimetype) + "' ESCAPE '\\'";
+        return QStringLiteral("mimetype LIKE '") + Common::starPatternToLike(mimetype) + QStringLiteral("' ESCAPE '\\'");
+    }
+
+    QString dateClause(QDate date) const {
+        return QStringLiteral("DATE(re.start, 'unixepoch') = '") +
+                               date.toString(Qt::ISODate) + QStringLiteral("' ");
+    }
+
+    QString resourceEventJoinClause() const {
+        return QStringLiteral(R"sql(
+            LEFT JOIN
+                ResourceEvent re
+                ON  from_table.targettedResource = re.targettedResource
+                AND from_table.usedActivity      = re.usedActivity
+                AND from_table.initiatingAgent   = re.initiatingAgent
+        )sql");
     }
 
     /**
@@ -205,11 +218,11 @@ public:
 
         const int limit = queryDefinition.limit();
         if (limit > 0) {
-            result += " LIMIT " + QString::number(limit);
+            result += QStringLiteral(" LIMIT ") + QString::number(limit);
 
             const int offset = queryDefinition.offset();
             if (offset > 0) {
-                result += " OFFSET " + QString::number(offset);
+                result += QStringLiteral(" OFFSET ") + QString::number(offset);
             }
         }
 
@@ -228,7 +241,6 @@ public:
               : QString()
             );
 
-
         // WHERE clause for filtering on agents
         QStringList agentsFilter = transformedList(
                 queryDefinition.agents(), &ResultSetPrivate::agentClause);
@@ -245,170 +257,257 @@ public:
         QStringList mimetypeFilter = transformedList(
                 queryDefinition.types(), &ResultSetPrivate::mimetypeClause);
 
-        auto query = _query
-            + "\nORDER BY $orderingColumn resource ASC\n"
-            + limitOffsetSuffix();
+        QString dateColumn = QStringLiteral("1"), resourceEventJoin;
+        // WHERE clause for access date filtering and ResourceEvent table Join
+        if (!queryDefinition.date().isNull()) {
+            dateColumn = dateClause(queryDefinition.date());
 
+            resourceEventJoin = resourceEventJoinClause();
+        }
+
+        auto queryString = _query;
+
+        queryString.replace(QStringLiteral("ORDER_BY_CLAUSE"), QStringLiteral("ORDER BY $orderingColumn resource ASC"))
+                   .replace(QStringLiteral("LIMIT_CLAUSE"), limitOffsetSuffix());
 
         return kamd::utils::debug_and_return(DEBUG_QUERIES, "Query: ",
-            query
+            queryString
                 .replace(QLatin1String("$orderingColumn"), orderingColumn)
                 .replace(QLatin1String("$agentsFilter"), agentsFilter.join(QStringLiteral(" OR ")))
                 .replace(QLatin1String("$activitiesFilter"), activitiesFilter.join(QStringLiteral(" OR ")))
                 .replace(QLatin1String("$urlFilter"), urlFilter.join(QStringLiteral(" OR ")))
                 .replace(QLatin1String("$mimetypeFilter"), mimetypeFilter.join(QStringLiteral(" OR ")))
+                .replace(QLatin1String("$resourceEventJoin"), resourceEventJoin)
+                .replace(QLatin1String("$dateFilter"), dateColumn)
             );
     }
 
-    const QString &linkedResourcesQuery() const
+    static const QString &linkedResourcesQuery()
     {
         // TODO: We need to correct the scores based on the time that passed
         //       since the cache was last updated, although, for this query,
         //       scores are not that important.
-        static const QString query =
-            R"sql(
+        static const QString queryString =
+            QStringLiteral(R"sql(
             SELECT
-                rl.targettedResource as resource
-              , SUM(rsc.cachedScore) as score
-              , MIN(rsc.firstUpdate) as firstUpdate
-              , MAX(rsc.lastUpdate)  as lastUpdate
-              , rl.usedActivity      as activity
-              , rl.initiatingAgent   as agent
-              , COALESCE(ri.title, rl.targettedResource) as title
+                from_table.targettedResource as resource
+              , SUM(rsc.cachedScore)         as score
+              , MIN(rsc.firstUpdate)         as firstUpdate
+              , MAX(rsc.lastUpdate)          as lastUpdate
+              , from_table.usedActivity      as activity
+              , from_table.initiatingAgent   as agent
+              , COALESCE(ri.title, from_table.targettedResource) as title
               , ri.mimetype as mimetype
-              , 2       as linkStatus
+              , 2 as linkStatus
 
             FROM
-                ResourceLink rl
+                ResourceLink from_table
             LEFT JOIN
                 ResourceScoreCache rsc
-                ON rl.targettedResource = rsc.targettedResource
-                AND rl.usedActivity     = rsc.usedActivity
-                AND rl.initiatingAgent  = rsc.initiatingAgent
+                ON  from_table.targettedResource = rsc.targettedResource
+                AND from_table.usedActivity      = rsc.usedActivity
+                AND from_table.initiatingAgent   = rsc.initiatingAgent
             LEFT JOIN
                 ResourceInfo ri
-                ON rl.targettedResource = ri.targettedResource
+                ON from_table.targettedResource = ri.targettedResource
+
+            $resourceEventJoin
 
             WHERE
                 ($agentsFilter)
                 AND ($activitiesFilter)
                 AND ($urlFilter)
                 AND ($mimetypeFilter)
+                AND ($dateFilter)
 
             GROUP BY resource, title
-            )sql"
+
+            ORDER_BY_CLAUSE
+            LIMIT_CLAUSE
+            )sql")
             ;
 
-        return query;
+        return queryString;
     }
 
-    const QString &usedResourcesQuery() const
+    static const QString &usedResourcesQuery()
     {
         // TODO: We need to correct the scores based on the time that passed
         //       since the cache was last updated
-        static const QString query =
-            R"sql(
+        static const QString queryString =
+            QStringLiteral(R"sql(
             SELECT
-                rsc.targettedResource as resource
-              , SUM(rsc.cachedScore)  as score
-              , MIN(rsc.firstUpdate)  as firstUpdate
-              , MAX(rsc.lastUpdate)   as lastUpdate
-              , rsc.usedActivity      as activity
-              , rsc.initiatingAgent   as agent
-              , COALESCE(ri.title, rsc.targettedResource) as title
+                from_table.targettedResource as resource
+              , SUM(from_table.cachedScore)  as score
+              , MIN(from_table.firstUpdate)  as firstUpdate
+              , MAX(from_table.lastUpdate)   as lastUpdate
+              , from_table.usedActivity      as activity
+              , from_table.initiatingAgent   as agent
+              , COALESCE(ri.title, from_table.targettedResource) as title
               , ri.mimetype as mimetype
-              , 1 as linkStatus -- Note: this is replaced by allResourcesQuery
+              , 1 as linkStatus
 
             FROM
-                ResourceScoreCache rsc
+                ResourceScoreCache from_table
             LEFT JOIN
                 ResourceInfo ri
-                ON rsc.targettedResource = ri.targettedResource
+                ON from_table.targettedResource = ri.targettedResource
+
+            $resourceEventJoin
 
             WHERE
                 ($agentsFilter)
                 AND ($activitiesFilter)
                 AND ($urlFilter)
                 AND ($mimetypeFilter)
+                AND ($dateFilter)
 
             GROUP BY resource, title
-            )sql"
+
+            ORDER_BY_CLAUSE
+            LIMIT_CLAUSE
+            )sql")
             ;
 
-        return query;
+        return queryString;
     }
 
-    const QString &allResourcesQuery() const
+    static const QString &allResourcesQuery()
     {
-        // TODO: Implement counting of the linked items
+        // TODO: We need to correct the scores based on the time that passed
+        //       since the cache was last updated, although, for this query,
+        //       scores are not that important.
+        static const QString queryString =
+            QStringLiteral(R"sql(
+            WITH
+                LinkedResourcesResults AS (
+                    SELECT from_table.targettedResource as resource
+                         , rsc.cachedScore              as score
+                         , rsc.firstUpdate              as firstUpdate
+                         , rsc.lastUpdate               as lastUpdate
+                         , from_table.usedActivity      as activity
+                         , from_table.initiatingAgent   as agent
+                         , 2 as linkStatus
 
-        // int linkedItemsCount = 0;
-        //
-        // if (linkedItemsCount >= limit) {
-        //     return linkedResourcesQuery();
-        //
-        // } else if (linkedItemsCount == 0) {
-        //     return usedResourcesQuery();
-        //
-        // } else {
+                    FROM
+                        ResourceLink from_table
 
-            static QString usedResourcesQuery_ = usedResourcesQuery();
+                    LEFT JOIN
+                        ResourceScoreCache rsc
+                        ON  from_table.targettedResource = rsc.targettedResource
+                        AND from_table.usedActivity      = rsc.usedActivity
+                        AND rl.initiatingAgent   = rsc.initiatingAgent
 
-            static const QString query =
-                "WITH LinkedResourcesResults as (\n" +
-                linkedResourcesQuery() +
-                "\n)\n" +
-                "SELECT * FROM LinkedResourcesResults \n" +
-                "UNION \n" +
-                usedResourcesQuery_
-                    .replace(QLatin1String("WHERE "),
-                        QLatin1String("WHERE rsc.targettedResource NOT IN "
-                        "(SELECT resource FROM LinkedResourcesResults) AND "))
-                    .replace(QLatin1String("1 as linkStatus"), QLatin1String("0 as linkStatus"));
+                    $resourceEventJoin
 
-        // }
+                    WHERE
+                        ($agentsFilter)
+                        AND ($activitiesFilter)
+                        AND ($urlFilter)
+                        AND ($mimetypeFilter)
+                        AND ($dateFilter)
+                ),
 
-        return query;
+                UsedResourcesResults AS (
+                    SELECT from_table.targettedResource as resource
+                         , from_table.cachedScore       as score
+                         , from_table.firstUpdate       as firstUpdate
+                         , from_table.lastUpdate        as lastUpdate
+                         , from_table.usedActivity      as activity
+                         , from_table.initiatingAgent   as agent
+                         , 0 as linkStatus
+
+                    FROM
+                        ResourceScoreCache from_table
+
+                    $resourceEventJoin
+
+                    WHERE
+                        ($agentsFilter)
+                        AND ($activitiesFilter)
+                        AND ($urlFilter)
+                        AND ($mimetypeFilter)
+                        AND ($dateFilter)
+                ),
+
+                CollectedResults AS (
+                    SELECT *
+                    FROM LinkedResourcesResults
+
+                    UNION
+
+                    SELECT *
+                    FROM UsedResourcesResults
+                    WHERE resource NOT IN (SELECT resource FROM LinkedResourcesResults)
+                )
+
+                SELECT
+                    resource
+                  , SUM(score) as score
+                  , MIN(firstUpdate) as firstUpdate
+                  , MAX(lastUpdate) as lastUpdate
+                  , activity
+                  , agent
+                  , COALESCE(ri.title, resource) as title
+                  , ri.mimetype as mimetype
+                  , linkStatus
+
+                FROM CollectedResults cr
+
+                LEFT JOIN
+                    ResourceInfo ri
+                    ON cr.resource = ri.targettedResource
+
+                GROUP BY resource, title
+
+                ORDER_BY_CLAUSE
+                LIMIT_CLAUSE
+            )sql")
+            ;
+
+        return queryString;
     }
 
     ResultSet::Result currentResult() const
     {
         ResultSet::Result result;
+
+        if (!database || !query.isActive()) return result;
+
         result.setResource(query.value(QStringLiteral("resource")).toString());
         result.setTitle(query.value(QStringLiteral("title")).toString());
         result.setMimetype(query.value(QStringLiteral("mimetype")).toString());
         result.setScore(query.value(QStringLiteral("score")).toDouble());
-        result.setLastUpdate(query.value(QStringLiteral("lastUpdate")).toInt());
-        result.setFirstUpdate(query.value(QStringLiteral("firstUpdate")).toInt());
-
+        result.setLastUpdate(query.value(QStringLiteral("lastUpdate")).toUInt());
+        result.setFirstUpdate(query.value(QStringLiteral("firstUpdate")).toUInt());
 
         result.setLinkStatus(
-            (ResultSet::Result::LinkStatus)query.value(QStringLiteral("linkStatus")).toInt());
+            static_cast<ResultSet::Result::LinkStatus>(query.value(QStringLiteral("linkStatus")).toUInt()));
 
-        auto query = database->createQuery();
+        auto linkedActivitiesQuery = database->createQuery();
 
-        query.prepare(R"sql(
+        linkedActivitiesQuery.prepare(QStringLiteral(R"sql(
             SELECT usedActivity
             FROM   ResourceLink
             WHERE  targettedResource = :resource
-            )sql");
+            )sql"));
 
-        query.bindValue(":resource", result.resource());
-        query.exec();
+        linkedActivitiesQuery.bindValue(QStringLiteral(":resource"), result.resource());
+        linkedActivitiesQuery.exec();
 
         QStringList linkedActivities;
-        for (const auto &item: query) {
+        for (const auto &item: linkedActivitiesQuery) {
             linkedActivities << item[0].toString();
         }
 
         result.setLinkedActivities(linkedActivities);
-        qDebug() << result.resource() << "linked to activities" << result.linkedActivities();
+        // qDebug() << result.resource() << "linked to activities" << result.linkedActivities();
 
         return result;
     }
 };
 
-ResultSet::ResultSet(Query query)
+ResultSet::ResultSet(Query queryDefinition)
     : d(new ResultSetPrivate())
 {
     using namespace Common;
@@ -420,10 +519,9 @@ ResultSet::ResultSet(Query query)
                       "that you do not have the Activity Manager running, or that "
                       "something else is broken on your system. Recent documents and "
                       "alike will not work!";
-        Q_ASSERT_X((bool)d->database, "ResultSet constructor", "Database is NULL");
     }
 
-    d->queryDefinition = query;
+    d->queryDefinition = queryDefinition;
 
     d->initQuery();
 }
@@ -452,7 +550,7 @@ ResultSet::~ResultSet()
 
 ResultSet::Result ResultSet::at(int index) const
 {
-    Q_ASSERT_X(d->query.isActive(), "ResultSet::at", "Query is not active");
+    if (!d->query.isActive()) return Result();
 
     d->query.seek(index);
 
